@@ -1,18 +1,22 @@
 from typing import List
-from websocket import create_connection
+from websocket import create_connection, WebSocketConnectionClosedException
 from PIL import Image
 from io import BytesIO
 from collections import defaultdict
 from datetime import datetime
 from importlib import resources
+import threading
+import asyncio
 
 from .rustplus_pb2 import *
 from .structures import RustTime, RustInfo, RustMap, RustMarker, RustChatMessage, RustSuccess, RustTeamInfo, RustTeamMember, RustTeamNote, RustEntityInfo, RustContents, RustItem
 from ..utils import MonumentNameToImage, TimeParser, CoordUtil, ErrorChecker, IdToName, MapMarkerConverter
-from ..exceptions import ImageError, ServerNotResponsiveError
+from ..exceptions import ImageError, ServerNotResponsiveError, ClientNotConnectedError, CommandsNotEnabledError
+from ..commands import CommandOptions, RustCommandHandler
 
 class RustSocket:
-    def __init__(self, ip : str, port : str, steamid : int, playertoken : int) -> None:
+
+    def __init__(self, ip : str, port : str, steamid : int, playertoken : int, command_options : CommandOptions = None) -> None:
         
         self.seq = 1
         self.ip = ip
@@ -20,8 +24,18 @@ class RustSocket:
         self.steamid = steamid
         self.playertoken = playertoken
         self.error_checker = ErrorChecker()
+        self.responses = {}
+        self.ignored_responses = []
+        self.prefix = None
+        self.command_handler = None
+        self.ws = None
 
-    def __repr__(self) -> str:
+        if command_options is not None:
+
+            self.prefix = command_options.prefix
+            self.command_handler = RustCommandHandler(command_options)
+
+    def __str__(self) -> str:
         return "RustSocket[ip = {} | port = {} | steamid = {} | playertoken = {}]".format(self.ip, self.port, self.steamid, self.playertoken)
 
     def __initProto(self) -> AppRequest:
@@ -32,19 +46,64 @@ class RustSocket:
         request.playerToken = self.playertoken
         return request
 
-    async def __sendAndRecieve(self, request) -> AppMessage:
+    def __start_listener(self) -> None:
+
+        loop = asyncio.new_event_loop()
+
+        loop.run_until_complete(self.__listener())
+        loop.close()
+
+    async def __listener(self) -> None:
+
+        while self.ws != None:
+
+            try:
+
+                data = self.ws.recv()
+
+                app_message = AppMessage()
+                app_message.ParseFromString(data)
+
+                if app_message.broadcast.teamMessage.message.message == "":
+                    if app_message.response.seq not in self.ignored_responses:
+                        self.responses[app_message.response.seq] = app_message
+                    else:
+                        self.ignored_responses.remove(app_message.response.seq)
+                else:
+                    message = RustChatMessage(app_message.broadcast.teamMessage.message)
+                    
+                    if message.message.startswith(self.prefix):
+                        await self.command_handler.run_command(message=message)
+                    
+            except WebSocketConnectionClosedException:
+                return
+
+    async def __getResponse(self, seq):
+
+        while seq not in self.responses:
+            await asyncio.sleep(0.1)
+        response = self.responses[seq]
+        del self.responses[seq]
+        return response
+
+    async def __sendAndRecieve(self, request, response = True) -> AppMessage:
 
         data = request.SerializeToString()
+
+        if self.ws == None:
+            raise ClientNotConnectedError("Not Connected")
+
         self.ws.send_binary(data)
 
-        return_data = self.ws.recv()
+        if response:
 
-        app_message = AppMessage()
-        app_message.ParseFromString(return_data)
+            app_message = await self.__getResponse(request.seq)
 
-        await self.error_checker.check(app_message)
+            await self.error_checker.check(app_message)
 
-        return app_message
+            return app_message
+
+        return None
 
     async def __getTime(self) -> RustTime:
 
@@ -167,37 +226,11 @@ class RustSocket:
 
         request = self.__initProto()
         request.sendTeamMessage.CopyFrom(msg)
-        data = request.SerializeToString()
 
-        self.ws.send_binary(data)
+        self.ignored_responses.append(request.seq)
+        await self.__sendAndRecieve(request, False)
 
-        messageReturn = self.ws.recv()
-        messageReturnAppMessage = AppMessage()
-        messageReturnAppMessage.ParseFromString(messageReturn)
-        if str(messageReturnAppMessage.response.error) != "":
-            return messageReturnAppMessage
-        
-        success = self.ws.recv()
-
-        appMessage = AppMessage()
-        appMessage.ParseFromString(success)
-
-        await self.error_checker.check(appMessage)
-
-        return RustSuccess(appMessage.response.seq, appMessage.response.success)
-
-    async def __getCameraFrame(self, id, frame):
-
-        cameraPacket = AppCameraFrameRequest()
-        cameraPacket.identifier = id
-        cameraPacket.frame = frame
-
-        request = self.__initProto()
-        request.getCameraFrame.CopyFrom(cameraPacket)
-
-        app_message = await self.__sendAndRecieve(request)
-
-        return app_message
+        return RustSuccess(0,"Success")
 
     async def __getTeamInfo(self):
 
@@ -289,11 +322,14 @@ class RustSocket:
         except:
             raise ServerNotResponsiveError("The sever is not available to connect to - your ip/port are either correct or the server is offline")
 
+        threading.Thread(target=self.__start_listener).start()
+
     async def closeConnection(self) -> None:
         """
         Close the connection to the Rust Server
         """
         self.ws.abort()
+        self.ws = None
 
     async def disconnect(self) -> None:
         """
@@ -334,12 +370,7 @@ class RustSocket:
 
     async def getTeamChat(self) -> List[RustChatMessage]:
         """
-        Returns a list of chat messages, formatted as 'ChatMessage' objects with entries:
-        - steamId, 
-        - name,
-        - message, 
-        - colour
-        - time
+        Returns a list of RustChatMessage objects
         """
 
         return await self.__getTeamChat()
@@ -351,20 +382,6 @@ class RustSocket:
 
         return await self.__sendTeamChatMessage(message)
 
-    async def getCameraFrame(self, id : str, frame : int) -> Image:
-        """
-        Returns a low quality jpeg image from a camera in-game
-        """
-
-        returnData = await self.__getCameraFrame(id,frame)
-
-        try:
-            image = Image.open(BytesIO(returnData.response.cameraFrame.jpgImage))
-        except:
-            raise ImageError("Invalid Bytes Recieved")
-
-        return image
-
     async def getTeamInfo(self) -> RustTeamInfo:
         """
         Returns an AppTeamInfo object of the players in your team, as well as a lot of data about them
@@ -420,4 +437,18 @@ class RustSocket:
         Returns the MapMarker for the event
         """
         
-        return await self.__getCurrentEvents()
\ No newline at end of file
+        return await self.__getCurrentEvents()
+
+    def command(self, coro) -> None:
+        """A Decorator that registers an commands listener"""
+
+        if self.command_handler is None:
+            raise CommandsNotEnabledError("Commands have not been enabled in the constructor")
+            
+        self.command_handler.register_command(coro.__name__, coro)
+
+    async def hang(self) -> None:
+        """This Will permanently put your script into a state of 'hanging'. Only do this in scripts using commands"""
+
+        while True:
+            await asyncio.sleep(1)
